@@ -11,7 +11,8 @@ import logging
 
 from doorstop import CLI, VERSION
 from doorstop.cli import settings
-from doorstop.core import processor
+from doorstop.core.tree import build
+from doorstop.core import report
 from doorstop.common import DoorstopError
 
 
@@ -39,7 +40,7 @@ class _WarningFormatter(logging.Formatter, object):
         return super().format(record)
 
 
-def main(args=None):
+def main(args=None):  # pylint: disable=R0915
     """Process command-line arguments and run the program.
     """
     # Shared options
@@ -52,28 +53,28 @@ def main(args=None):
     # Main parser
     parser = argparse.ArgumentParser(prog=CLI, description=__doc__, **shared)
     parser.add_argument('-g', '--gui', action='store_true',
-                        help="launch the GUI")
+                        help="launch the graphical user interface")
     subs = parser.add_subparsers(help="", dest='command', metavar="<command>")
 
     # New subparser
     sub = subs.add_parser('new',
                           help="create a new document directory",
                           **shared)
-    sub.add_argument('prefix', help="prefix for item IDs")
-    sub.add_argument('root', help="path to directory for document items")
+    sub.add_argument('prefix', help="document prefix for new item IDs")
+    sub.add_argument('root', help="path to a directory for document items")
     sub.add_argument('-p', '--parent', help="prefix for parent item IDS")
     sub.add_argument('-d', '--digits', help="number of digits in item IDs")
 
     # Add subparser
     sub = subs.add_parser('add',
-                          help="add a new item to a document",
+                          help="add a new item to a document directory",
                           **shared)
     sub.add_argument('prefix',
-                     help="prefix of document for the new item")
+                     help="document prefix for the new item")
 
     # Remove subparser
     sub = subs.add_parser('remove',
-                          help="remove an item from a document",
+                          help="remove an item from a document directory",
                           **shared)
     sub.add_argument('id', metavar='ID',
                      help="item ID to remove from a document")
@@ -98,9 +99,9 @@ def main(args=None):
 
     # Edit subparser
     sub = subs.add_parser('edit',
-                          help="edit an existing document item",
+                          help="open an existing item file for editing",
                           **shared)
-    sub.add_argument('id', metavar='ID', help="item to edit")
+    sub.add_argument('id', metavar='ID', help="item ID to open for editing")
     sub.add_argument('-t', '--tool', metavar='PROGRAM',
                      help="text editor to open the document item")
 
@@ -116,11 +117,17 @@ def main(args=None):
                           **shared)
     sub.add_argument('output', help="file to export")
 
-    # Report subparser
-    sub = subs.add_parser('report',
-                          help="publish document items to a report",
+    # Publish subparser
+    sub = subs.add_parser('publish',
+                          help="publish a document as text or another format",
                           **shared)
-    sub.add_argument('report', help="report to create")
+    sub.add_argument('prefix', help="prefix of document to publish")
+    sub.add_argument('-m', '--markdown', action='store_true',
+                     help="output Markdown instead of raw text")
+    sub.add_argument('-H', '--html', action='store_true',
+                     help="output HTML converted from Markdown")
+    sub.add_argument('-w', '--width', type=int,
+                     help="limit line width on text output")
 
     # Parse arguments
     args = parser.parse_args(args=args)
@@ -178,13 +185,13 @@ def _run(args, cwd, err):  # pylint: disable=W0613
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
+        tree = build(cwd)
         tree.check()
     except DoorstopError as error:
         logging.error(error)
         return False
     else:
-        print("tree: {}".format(tree))
+        print("validated: {}".format(tree))
         return True
 
 
@@ -195,14 +202,14 @@ def _run_new(args, cwd, _):
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
-        document = tree.new(args.root, args.prefix,
-                            parent=args.parent, digits=args.digits)
-        print("created: {}".format(document))
+        tree = build(cwd)
+        doc = tree.new(args.root, args.prefix,
+                       parent=args.parent, digits=args.digits)
     except DoorstopError as error:
         logging.error(error)
         return False
     else:
+        print("created: {}".format(doc))
         return True
 
 
@@ -213,7 +220,7 @@ def _run_add(args, cwd, _):
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
+        tree = build(cwd)
         item = tree.add(args.prefix)
     except DoorstopError as error:
         logging.error(error)
@@ -230,7 +237,7 @@ def _run_remove(args, cwd, _):
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
+        tree = build(cwd)
         item = tree.remove(args.id)
     except DoorstopError as error:
         logging.error(error)
@@ -247,7 +254,7 @@ def _run_link(args, cwd, _):
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
+        tree = build(cwd)
         child, parent = tree.link(args.child, args.parent)
     except DoorstopError as error:
         logging.error(error)
@@ -264,7 +271,7 @@ def _run_unlink(args, cwd, _):
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
+        tree = build(cwd)
         child, parent = tree.unlink(args.child, args.parent)
     except DoorstopError as error:
         logging.error(error)
@@ -281,7 +288,7 @@ def _run_edit(args, cwd, _):
     @param err: function to call for CLI errors
     """
     try:
-        tree = processor.build(cwd)
+        tree = build(cwd)
         item = tree.edit(args.id, tool=args.tool, launch=True)
     except DoorstopError as error:
         logging.error(error)
@@ -311,14 +318,37 @@ def _run_export(args, cwd, err):
     raise NotImplementedError("'doorstop export' not implemented")
 
 
-def _run_report(args, cwd, err):
+def _run_publish(args, cwd, _):
     """Process arguments and run the `doorstop report` subcommand.
     @param args: Namespace of CLI arguments
     @param cwd: current working directory
     @param err: function to call for CLI errors
     """
-    logging.warning((args, cwd, err))
-    raise NotImplementedError("'doorstop report' not implemented")
+    try:
+        tree = build(cwd)
+        doc = tree.find_document(args.prefix)
+    except DoorstopError as error:
+        logging.error(error)
+        return False
+    else:
+
+        kwargs = {'ignored': tree.vcs.ignored}
+
+        if args.markdown:
+            for line in report.get_markdown(doc, **kwargs):
+                print(line)
+
+        elif args.html:
+            for line in report.get_html(doc, **kwargs):
+                print(line)
+
+        else:  # raw text
+            if args.width:
+                kwargs['width'] = args.width
+            for line in report.get_text(doc, **kwargs):
+                print(line)
+
+        return True
 
 
 if __name__ == '__main__':  # pragma: no cover, manual test
