@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """
 Representation of items in a Doorstop document.
 """
@@ -15,6 +13,10 @@ import yaml
 from doorstop.common import DoorstopError
 
 
+# http://en.wikipedia.org/wiki/Sentence_boundary_disambiguation
+SBD = re.compile(r"((?<=[a-z0-9][.?!])|(?<=[a-z0-9][.?!]\"))(\s|\r\n)(?=\"?[A-Z])")  # pylint: disable=C0301
+
+
 class _literal(str):  # pylint: disable=R0904
     """Custom type for text which should be dumped in the literal style."""
     pass
@@ -22,7 +24,8 @@ class _literal(str):  # pylint: disable=R0904
 
 def _literal_representer(dumper, data):
     """Return a custom dumper that formats str in the literal style."""
-    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style='|')
+    style = '|' if data else ''
+    return dumper.represent_scalar('tag:yaml.org,2002:str', data, style=style)
 
 yaml.add_representer(_literal, _literal_representer)
 
@@ -49,7 +52,6 @@ def _auto_save(func):
     return wrapped
 
 
-# TODO: only load if an attribute is blank?
 class Item(object):
     """Represents a file with linkable text that is part of a document."""
 
@@ -57,7 +59,7 @@ class Item(object):
 
     DEFAULT_LEVEL = (1,)
     DEFAULT_TEXT = ""
-    DEFAULT_REF = None
+    DEFAULT_REF = ""
     DEFAULT_LINKS = set()
 
     def __init__(self, path, root=os.getcwd(),
@@ -146,23 +148,26 @@ class Item(object):
         with open(path, 'w'):
             pass  # just touch the file
 
-    def load(self):
+    def load(self, reload=False):
         """Load the item's properties from a file."""
+        if getattr(self, '_loaded', False) and not reload:
+            return
         logging.debug("loading {}...".format(repr(self)))
         # Read the YAML from file
         text = self._read()
         # Parse the YAML data
         try:
             data = yaml.load(text)
-        except yaml.scanner.ScannerError as error:
+        except yaml.scanner.ScannerError as error:  # pylint: disable=E1101
             msg = "invalid contents: {}:\n{}".format(self, error)
             raise DoorstopError(msg)
         # Store parsed data
         if data:
             self._level = self._convert_level(data.get('level', self._level))
-            self._text = data.get('text', self._text).strip()
+            self._text = data.get('text', self._text)
             self._ref = data.get('ref', self._ref)
             self._links = set(data.get('links', self._links))
+        setattr(self, '_loaded', True)
 
     def _read(self):  # pragma: no cover, integration test
         """Read text from the file."""
@@ -172,7 +177,7 @@ class Item(object):
             return infile.read().decode('UTF-8')
 
     def save(self):
-        """Save the item's properties to a file."""
+        """Format and save the item's properties to a file."""
         logging.debug("saving {}...".format(repr(self)))
         # Collect the data items
         level = '.'.join(str(n) for n in self._level)
@@ -180,19 +185,32 @@ class Item(object):
             level = float(level)
         elif len(self._level) == 1:
             level = int(level)
-        text = _literal(self._text)
-        ref = self._ref
+        text = _literal(self._sbd(self._text))
+        ref = self._ref.strip()
         links = sorted(self._links)
         # Build the data structure
         data = {'level': level,
                 'text': text,
-                'links': links}
-        if ref:
-            data['ref'] = ref
+                'links': links,
+                'ref': ref}
         # Dump the data to YAML
         dump = yaml.dump(data, default_flow_style=False)
         # Save the YAML to file
         self._write(dump)
+        setattr(self, '_loaded', False)
+
+    @staticmethod
+    def _sbd(text):
+        """Replace sentence boundaries with newlines.
+
+        >>> Item._sbd("Hello, world!")
+        'Hello, world!'
+
+        >>> Item._sbd("Hello, world! How are you? I'm fine. Good.")
+        "Hello, world!\\nHow are you?\\nI'm fine.\\nGood."
+
+        """
+        return SBD.sub('\n', text.strip())
 
     def _write(self, text):  # pragma: no cover, integration test
         """Write text to the file."""
@@ -212,6 +230,7 @@ class Item(object):
 
         >>> Item.join_id('ABC', 5, 123)
         'ABC00123'
+
         """
         return "{}{}".format(prefix, str(number).zfill(digits))
 
@@ -250,6 +269,14 @@ class Item(object):
         """Set the item's level."""
         self._level = self._convert_level(level)
 
+    @property
+    def heading(self):
+        """Get the heading order based on the level."""
+        level = list(self.level)
+        while level[-1] == 0:
+            del level[-1]
+        return len(level)
+
     @staticmethod
     def _convert_level(text):
         """Convert a level string to a tuple.
@@ -284,7 +311,7 @@ class Item(object):
     @_auto_save
     def text(self, text):
         """Set the item's text."""
-        self._text = text.strip()
+        self._text = text
 
     @property
     @_auto_load
@@ -299,21 +326,45 @@ class Item(object):
         self._ref = ref
 
     @_auto_load
-    def find_ref(self):
-        """Find the external file reference and line number."""
+    def find_ref(self, root=None, ignored=None):
+        """Find the external file reference and line number.
+
+        @param root: override the path to the working copy (for testing)
+        @param ignored: function to determine if a path should be skipped
+
+        @return: path to file, line number (or None, None when no ref)
+
+        @raise DoorstopError: when no ref is found
+        """
         if not self.ref:
             logging.debug("no external reference to search for")
             return None, None
-        regex = re.compile(r"\b{}\b".format(self.ref))
-        for root, _, filenames in os.walk(os.path.dirname(self.path)):
+        ignored = ignored or (lambda _: False)
+        logging.info("seraching for ref '{}'...".format(self.ref))
+        pattern = r"(\b|\W){}(\b|\W)".format(re.escape(self.ref))
+        logging.debug("regex: {}".format(pattern))
+        regex = re.compile(pattern)
+        for root, _, filenames in os.walk(root or self.root):
             for filename in filenames:  # pragma: no cover, integration test
                 path = os.path.join(root, filename)
+                # Skip the item's file while searching
                 if path == self.path:
                     continue
-                with open(path) as external:
-                    for index, line in enumerate(external):
-                        if regex.search(line):
-                            return path, index + 1
+                # Skip hidden directories
+                if (os.path.sep + '.') in path:
+                    continue
+                # Skip ignored paths
+                if ignored(path):
+                    continue
+                # Search for the reference in the file
+                try:
+                    with open(path, 'r') as external:
+                        for index, line in enumerate(external):
+                            if regex.search(line):
+                                logging.info("found ref: {}".format(path))
+                                return path, index + 1
+                except UnicodeDecodeError:
+                    pass
         msg = "external reference not found: {}".format(self.ref)
         raise DoorstopError(msg)
 
@@ -344,17 +395,40 @@ class Item(object):
         except KeyError:
             logging.warning("link to {0} does not exist".format(item))
 
-    def check(self):
+    def check(self, document=None, tree=None, ignored=None):
         """Confirm the item is valid.
 
+        @param document: document to validate the item
+        @param tree: tree to validate the item
+        @param ignored: function to determine if a path should be skipped
+
         @return: indication that the item is valid
         """
         logging.info("checking item {}...".format(self))
         # Verify the file can be parsed
         self.load()
         # Check text
-        if not self.text:
+        if not self.text and not self.ref:
             logging.warning("no text: {}".format(self))
+        # Check external references
+        self.find_ref(ignored=ignored)
+        # Check links against the document
+        if document:
+            if document.parent and self.level[-1] != 0 and not self.links:
+                logging.warning("no links: {}".format(self))
+
+            for identifier in self.links:
+                prefix = self.split_id(identifier)[0]
+                if prefix.lower() != document.parent.lower():
+                    msg = "link to non-parent '{}' in {}".format(identifier,
+                                                                 self)
+                    logging.warning(msg)
+        # Check links against the tree
+        if tree:
+            for identifier in self.links:
+                item = tree.find_item(identifier)
+                logging.debug("found linked item: {}".format(item))
+
         # Reformat the file
         self.save()
         # Item is valid
