@@ -13,7 +13,7 @@ import logging
 from doorstop.common import DoorstopError
 from doorstop.core.item import Item
 
-from doorstop.core.test import FILES, EMPTY
+from doorstop.core.test import ENV, REASON, FILES, EMPTY
 
 
 class MockItem(Item):
@@ -21,8 +21,8 @@ class MockItem(Item):
 
     @patch('os.path.isfile', Mock(return_value=True))
     def __init__(self, *args, **kwargs):
+        self._file = kwargs.pop('_file', "")  # file system mock
         super().__init__(*args, **kwargs)
-        self._file = ""  # file system mock
         self._read = Mock(side_effect=self._mock_read)
         self._write = Mock(side_effect=self._mock_write)
 
@@ -45,8 +45,10 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
 
     def setUp(self):
         self.path = os.path.join('path', 'to', 'RQ001.yml')
-        self.item = MockItem(self.path)
-        self.item._file = "links: []\ntext: ''\nlevel: 1.1.1"
+        self.item = MockItem(self.path, _file=("links: []\n"
+                                               "ref: ''\n"
+                                               "text: ''\n"
+                                               "level: 1.1.1"))
 
     def test_load_empty(self):
         """Verify loading calls read."""
@@ -65,7 +67,10 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
     def test_save_empty(self):
         """Verify saving calls write."""
         self.item.save()
-        text = 'level: 1\nlinks: []\ntext: ""\n'
+        text = ("level: 1\n"
+                "links: []\n"
+                "ref: ''\n"
+                "text: ''\n")
         self.item._write.assert_called_once_with(text)
 
     def test_str(self):
@@ -77,7 +82,6 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         """Verify item non-equality is correct."""
         self.assertNotEqual(self.item, None)
 
-    # TODO: use MockItem.new()
     def test_lt(self):
         """Verify items can be compared."""
         item0 = MockItem('path/to/RQ002.yml', _level=(1, 1))
@@ -105,53 +109,71 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
     def test_level(self):
         """Verify an item's level can be set and read."""
         self.item.level = (1, 2, 3)
+        self.assertIn("level: 1.2.3\n", self.item._write.call_args[0][0])
         self.assertEqual((1, 2, 3), self.item.level)
-        text = 'level: 1.2.3\nlinks: []\ntext: ""\n'
-        self.item._write.assert_called_once_with(text)
+
+    def test_heading(self):
+        """Verify the heading can be read from the item's level."""
+        self.item.level = (1,)
+        self.assertEqual(1, self.item.heading)
+        self.item.level = (1, 0)
+        self.assertEqual(1, self.item.heading)
+        self.item.level = (2, 0, 1)
+        self.assertEqual(3, self.item.heading)
+        self.item.level = (2, 0, 1, 1, 0, 0)
+        self.assertEqual(4, self.item.heading)
 
     def test_level_from_text(self):
         """Verify an item's level can be set from text and read."""
-        self.item.level = "4.2"
-        self.assertEqual((4, 2), self.item.level)
-        text = 'level: 4.2\nlinks: []\ntext: ""\n'
-        self.item._write.assert_called_once_with(text)
+        self.item.level = "4.2.0 "
+        self.assertIn("level: 4.2.0\n", self.item._write.call_args[0][0])
+        self.assertEqual((4, 2, 0), self.item.level)
 
     def test_level_from_float(self):
         """Verify an item's level can be set from a float and read."""
         self.item.level = 4.2
+        self.assertIn("level: 4.2\n", self.item._write.call_args[0][0])
         self.assertEqual((4, 2), self.item.level)
-        text = 'level: 4.2\nlinks: []\ntext: ""\n'
-        self.item._write.assert_called_once_with(text)
 
     def test_level_from_int(self):
         """Verify an item's level can be set from a int and read."""
         self.item.level = 42
+        self.assertIn("level: 42\n", self.item._write.call_args[0][0])
         self.assertEqual((42,), self.item.level)
-        text = 'level: 42\nlinks: []\ntext: ""\n'
-        self.item._write.assert_called_once_with(text)
 
     def test_text(self):
         """Verify an item's text can be set and read."""
-        self.item.text = "test text"
-        self.assertEqual("test text", self.item.text)
+        self.item.text = "test "
+        self.assertIn("text: |-\n  test\n", self.item._write.call_args[0][0])
+        self.assertEqual("test", self.item.text)
+
+    def test_text_sentences(self):
+        """Verify newlines separate sentences in an item's text."""
+        self.item.text = ("A sentence. Another sentence! Hello? Hi.\n"
+                          "A new line. And another sentece.")
+        expected = ("A sentence.\n"
+                    "Another sentence!\n"
+                    "Hello?\n"
+                    "Hi.\n"
+                    "A new line.\n"
+                    "And another sentece.")
+        self.assertEqual(expected, self.item.text)
 
     def test_ref(self):
         """Verify an item's reference can be set and read."""
         self.item.ref = "abc123"
+        self.assertIn("ref: abc123\n", self.item._write.call_args[0][0])
         self.assertEqual("abc123", self.item.ref)
-        text = 'level: 1\nlinks: []\nref: abc123\ntext: ""\n'
-        self.item._write.assert_called_once_with(text)
-
-    def test_ref_none(self):
-        """Verify item refs are only saved if they exist."""
-        self.item.ref = None
-        self.assertEqual(None, self.item.ref)
-        text = 'level: 1\nlinks: []\ntext: ""\n'
-        self.item._write.assert_called_once_with(text)
 
     def test_find_ref_error(self):
-        """Verify an error is raised when no external reference is found."""
-        self.item.ref = "notfound"
+        """Verify an error occurs when no external reference found."""
+        self.item.ref = "not found".replace(' ', '')  # avoids self match
+        self.assertRaises(DoorstopError, self.item.find_ref, root=EMPTY)
+
+    @unittest.skipUnless(os.getenv(ENV), REASON)
+    def test_find_ref_error_long(self):
+        """Verify an error occurs when no external reference found (long)."""
+        self.item.ref = "not found".replace(' ', '')  # avoids self match
         self.assertRaises(DoorstopError, self.item.find_ref)
 
     def test_find_ref_none(self):
@@ -201,8 +223,33 @@ class TestItem(unittest.TestCase):  # pylint: disable=R0904
         self.assertRaises(DoorstopError,
                           Item.new, FILES, FILES, 'REQ', 3, 2, (1, 2, 3))
 
+    def test_check_document(self):
+        """Verify an item can be checked against a document."""
+        document = Mock()
+        document.parent = 'fake'
+        self.item.check(document=document)
+
+    def test_check_document_with_links(self):
+        """Verify an item can be checked against a document with links."""
+        self.item.add_link('unknown1')
+        document = Mock()
+        document.parent = 'fake'
+        self.item.check(document=document)
+
+    def test_check_tree(self):
+        """Verify an item can be checked against a tree."""
+        self.item.add_link('fake1')
+        tree = Mock(find_item=Mock())
+        self.item.check(tree=tree)
+
+    def test_check_tree_error(self):
+        """Verify an item can be checked against a tree with errors."""
+        self.item.add_link('fake1')
+        tree = Mock(find_item=Mock(side_effect=DoorstopError))
+        self.assertRaises(DoorstopError, self.item.check, tree=tree)
+
     @patch('os.remove')
-    def test_remove(self, mock_remove):
+    def test_delete(self, mock_remove):
         """Verify an item can be deleted."""
         self.item.delete()
         mock_remove.assert_called_once_with(self.item.path)
